
package  {
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Spritemap;
	
	/**
	 * ...
	 * @author D Kvarfordt
	 */
	public class Bird extends Entity {
		
		private var sprite:Spritemap = new Spritemap(Assets.BIRD_IMG,32,32);
		
		private var vx:Number;
		private var vy:Number;
		private var lastX:Number;
		private var lastY:Number;
		
		private var startX:Number;
		private var startY:Number;
		private var oppX:Number;
		private var oppY:Number;
		private var startDir:int;
		
		private var player:Player;
		
		private static const TURN_INTERVAL:int = 300;
		private static const RAY_INTERVAL:int = 30;
		private static const NORM_ACC:Number = 0.03;
		private static const NORM_DEACC:Number = 0.13;
		private static const NORM_MAXSPEED:Number = 1.5;
		private static const SPOTTED_WAIT:int = 100;
		private static const ATTACK_SPEED:int = 2.3;
		private static const ATTACK_TIME:int = 200;
		private static const UNSTUCK_TIME:int = 100;
		private static const RETURN_SPEED:int = 2;
		private static const SIGHT_DISTANCE:int = 100;
		
		private static const SITTING:int = -1;
		private static const NORMAL:int = 0;
		private static const SPOTTED:int = 1;
		private static const ATTACKING:int = 2;
		private static const HURT:int = 3;
		private static const RETURNING:int = 4;
		private static const UNSTUCK:int = 5;
		private static const DEAD:int = 6;
		
		private var state:int;
		private var dir:int;
		private var clearLOSPlayer:Boolean;
		
		private var turnTimer:int;
		private var spottedTimer:int;
		private var rayTimer:int;
		private var attackTimer:int;
		private var unstuckTimer:int;
		
		public function Bird(x:Number, y:Number, sit:Boolean=true, dir:int = -1) {
			this.x = x+5;
			this.y = y+3;
			this.startX = this.x;
			this.startY = this.y;
			this.oppX = 0;
			this.oppY = 0;
			this.vx = 0;
			this.vy = 0;
			this.state = NORMAL;
			this.dir = this.startDir = dir;
			this.turnTimer = TURN_INTERVAL;
			this.player = GameWorld.theWorld.player;
			this.clearLOSPlayer = false;
			
			if (sit) {
				state = SITTING;
			}
			
			sprite.add("fly", [0, 1, 2, 3, 4, 5], 15);
			sprite.add("hover", [6, 7, 8, 9, 10, 11], 15);
			sprite.add("attack", [1]);
			sprite.add("dead", [12], 1);
			sprite.add("sit", [13], 1);
			sprite.play("fly");
			sprite.originX = 16;
			sprite.originY = 16;
			sprite.flipped = (dir == -1);
			graphic = sprite;
			
			type = "bird";
			
			layer = -1;
			
			setHitbox(10, 13, -8, -9);
			centerOrigin();
		}
		
		override public function update():void {
			super.update();
			
			if (vx > 0) {
				sprite.flipped = false;
			}
			if (vx < 0) {
				sprite.flipped = true;
			}
			
			if (state == DEAD) {
				sprite.play("dead");
				vy += GameWorld.GRAVITY;
				moveBy(vx, vy, "solid");
				if (collide("solid", x, y + 1)) {
					vx *= 0.7;
					vy = 0;
				}
				type = "bird";
			}
			
			// In the same screen as the player, shoot a ray 
			if (rayTimer <= 0 
				&& Math.floor(x / GameWorld.SCREEN_WIDTH) == GameWorld.theWorld.currScrX
				&& Math.floor(y / GameWorld.SCREEN_HEIGHT) == GameWorld.theWorld.currScrY) {
				FP.world.add(new Ray(x, y, player, seesCallback, seesNotCallback));
				rayTimer = RAY_INTERVAL;
			} else {
				rayTimer--;
			}
			
			var distPlayer:Number = FP.distance(x, y, player.x, player.y);
			
			if (state == SITTING) {
				sprite.play("sit");
				type = "bird";
				if (clearLOSPlayer && distPlayer < SIGHT_DISTANCE && dir == FP.sign(player.x - x)) {
					Sounds.craa.play();
					state = SPOTTED;
					spottedTimer = SPOTTED_WAIT;
					vy -= 1;
				}
			} else if (state == NORMAL) {
				sprite.play("fly");
				type = "bird";
				vy = 0;
				
				// Turning
				if (turnTimer <= 0) {
					dir *= -1;
					turnTimer = TURN_INTERVAL;
				} else {
					turnTimer--;
				}
				
				if (collide("solid", x + dir * 40, y)) {
					dir *= -1;
					turnTimer = TURN_INTERVAL;
				}
				
				if (FP.sign(vx) == dir) {
					vx += dir * NORM_ACC;
				} else {
					vx += dir * NORM_DEACC;
				}
				
				if (vx > NORM_MAXSPEED) {
					vx = NORM_MAXSPEED;
				} else if (vx < -NORM_MAXSPEED) {
					vx = -NORM_MAXSPEED;
				}
				
				//sprite.originY = Math.abs(vx*2);
				
				if (clearLOSPlayer && distPlayer < SIGHT_DISTANCE && dir == FP.sign(player.x - x)) {
					Sounds.craa.play();
					state = SPOTTED;
					spottedTimer = SPOTTED_WAIT;
				}
				
			} else if (state == SPOTTED) {
				vx *= 0.3;
				vy *= 0.96;
				sprite.play("hover");
				type = "bird";
				if (!clearLOSPlayer) {
					state = RETURNING;
				}
				if (spottedTimer <= 0) {
					if (clearLOSPlayer) {
						state = ATTACKING;
						attackTimer = ATTACK_TIME;
					} else {
						state = RETURNING;
					}
					
				} else {
					spottedTimer--;
				}
			} else if (state == ATTACKING) {
				sprite.play("attack");
				type = "attackingbird";
				
				var dx:Number = player.x - x;
				var dy:Number = player.y - y;
				
				var dist:Number = Math.sqrt(dx * dx + dy * dy)+0.1;
				vx = dx / dist * ATTACK_SPEED;
				vy = dy / dist * ATTACK_SPEED;
				
				if (attackTimer <= 0) {
					state = RETURNING;
					//sprite.angle = 0;
				} else {
					attackTimer--;
				}
				
				//sprite.angle = -Math.atan2(dy, dx)*180/Math.PI + ((sprite.flipped)?180:0);
				
			} else if (state == RETURNING) {
				sprite.play("fly");
				type = "bird";
				var dx2:Number = startX - x;
				var dy2:Number = startY - y;
				
				var dist2:Number = Math.sqrt(dx2 * dx2 + dy2 * dy2)+0.1;
				vx = dx2 / dist2 * RETURN_SPEED;
				vy = dy2 / dist2 * RETURN_SPEED;
				
				if (lastX == x && lastY == y) {
					// stuck
					trace("raven stuck");
					state = UNSTUCK;
				}
				
				if (dist2 < 4) {
					state = SITTING;
					vx = 0;
					vy = 0;
					x = startX;
					y = startY;
					dir = startDir;
					sprite.flipped = (dir == -1);
				}
			} else if (state == UNSTUCK) {
				sprite.play("fly");
				type = "bird";
				
				var dx3:Number = startX - x;
				var dy3:Number = startY - y;
				
				var dist3:Number = Math.sqrt(dx3 * dx3 + dy3 * dy3)+0.1;
				
				if (oppX == 0 || oppY == 0) {
					// "freshly" stuck
					oppX = x + dx3 / dist3 + (Math.random() - 0.5) * dy3 * 5;
					oppY = y + dx3 / dist3 + (Math.random() - 0.5) * dx3 * 5;
					unstuckTimer = UNSTUCK_TIME*Math.random();
				}
				unstuckTimer--;
				
				var dx4:Number = oppX - x;
				var dy4:Number = oppY - y;
				
				var dist4:Number = Math.sqrt(dx4 * dx4 + dy4 * dy4)+0.1;
				
				vx = dx4 / dist4 * RETURN_SPEED;
				vy = dy4 / dist4 * RETURN_SPEED;
				
				if (dist4 < 20 || unstuckTimer <= 0) {
					oppX = 0;
					oppY = 0;
					state = RETURNING;
				}
				
				if (lastX == x && lastY == y) {
					// try again
					oppX = 0;
					oppY = 0;
				}
				
			}
			
			lastX = x;
			lastY = y;
			moveBy(vx, vy, "solid");
			
			if (collide("claw", x, y)) {
				state = DEAD;
				Sounds.scratch.play(0.5);
				// TODO: impact force
			} else if (state == ATTACKING && collide("player", x, y)) {
				attackTimer = 0;
				Sounds.pick.play();
			}
			
			if (state == ATTACKING && collide("player", x, y)) {
				state = RETURNING;
			}
		}
		
		private function seesCallback():void {
			if (FP.sign(player.x-x) == dir) {
				clearLOSPlayer = true;
			}
		}
		private function seesNotCallback():void {
			clearLOSPlayer = false;
		}
		
	}
	
}

