package  {
	import flash.display.Bitmap;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Backdrop;
	import net.flashpunk.graphics.Tilemap;
	import net.flashpunk.masks.Grid;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import net.flashpunk.World;
	
	/**
	 * ...
	 * @author D Kvarfordt
	 */
	public class GameWorld extends World {
		
		public static var theWorld:GameWorld;
		public static const GRAVITY:Number = 0.3;
		
		public static const TILE_SIZE:uint = 10;
		public static const TILES_PER_SCREEN_X:uint = 32;
		public static const TILES_PER_SCREEN_Y:uint = 24;
		public static const SCREEN_WIDTH:uint = TILES_PER_SCREEN_X * TILE_SIZE;
		public static const SCREEN_HEIGHT:uint = TILES_PER_SCREEN_Y * TILE_SIZE;
		
		public static const WORLD_WIDTH_SCREENS:uint = 18;
		public static const WORLD_HEIGHT_SCREENS:uint = 4;
		
		
		public var player:Player;
		private var main:Main;
		private var background:Background;
		private var tileMap:Tilemap;
		private var tileGrid:Grid;
		private var healthBar:HealthStatus;
		private var orbCounter:OrbStatus;
		
		public var currScrX:int = 0;
		public var currScrY:int = 0;
		
		public var windSpeed:Number = 1.4;
		public var rainAmount:Number = 1.1;
		private var targetWind:Number = 1.4;
		
		private var camOnPlayer:Boolean = true;
		
		private var messageQueue:Array;
		private var currentMessage:MessagePopup;
		
		private var numOrbs:int;
		
		private var won:Boolean;
		private var winTimer:int;
		
		public function GameWorld(main:Main, player:Player) {
			background = new Background();
			tileMap = new Tilemap(Assets.TILES_IMG, WORLD_WIDTH_SCREENS * TILES_PER_SCREEN_X * TILE_SIZE, WORLD_HEIGHT_SCREENS * TILES_PER_SCREEN_Y * TILE_SIZE, TILE_SIZE, TILE_SIZE);
			tileGrid = new Grid(WORLD_WIDTH_SCREENS * TILES_PER_SCREEN_X * TILE_SIZE, WORLD_HEIGHT_SCREENS * TILES_PER_SCREEN_Y * TILE_SIZE, TILE_SIZE, TILE_SIZE);
			healthBar = new HealthStatus();
			orbCounter = new OrbStatus();
			this.player = player;
			this.main = main;
			
			add(background);
			addGraphic(tileMap);
			
			add(player);
			addMask(tileGrid, "solid");
			add(healthBar);
			add(orbCounter);
			
			FP.camera.x = Math.floor(player.x / SCREEN_WIDTH) * SCREEN_WIDTH;
			FP.camera.y = Math.floor(player.y / SCREEN_HEIGHT) * SCREEN_HEIGHT;
			
			init();
		}
		
		public function init():void {
			messageQueue = new Array();
			theWorld = this;
			
			loadWorld(tileMap, tileGrid);
			player.init();
			positionPlayer();
			
			numOrbs = classCount(Orb);
			trace("numorbs:" + numOrbs);
			
			won = false;
			winTimer = 1000;
		}
		
		
		public function loadWorld(tileMap:Tilemap, tileGrid:Grid):void {
			
			var lvlBm:Bitmap = Bitmap(new Assets.LEVEL_IMG);
			var tempTileMap:Tilemap = new Tilemap(Assets.TILES_IMG, WORLD_WIDTH_SCREENS * TILES_PER_SCREEN_X * TILE_SIZE, WORLD_HEIGHT_SCREENS * TILES_PER_SCREEN_Y * TILE_SIZE, TILE_SIZE, TILE_SIZE);
			
			for (var i:int = 0; i < lvlBm.width; i++) {
				for (var j:int = 0; j < lvlBm.height; j++) {
					tempTileMap.setTile(i, j, colorToIndex(lvlBm.bitmapData.getPixel32(i, j)));
					if (SOLIDS.indexOf(tempTileMap.getTile(i,j)) != -1) {
						tileGrid.setTile(i, j, colorToIndex(lvlBm.bitmapData.getPixel32(i, j)) != 0);
					}
					maybeSpawn(lvlBm.bitmapData.getPixel32(i, j), i, j);
				}
			}
			
			// Fix tiling
			for (var i2:int = 0; i2 < lvlBm.width; i2++) {
				for (var j2:int = 0; j2 < lvlBm.height; j2++) {
					if (tempTileMap.getTile(i2,j2) != 0) {
						tileMap.setTile(i2, j2, autoTile(tempTileMap,i2,j2));
					} else {
						tileMap.setTile(i2, j2, 0);
					}
				}
			}
			
			//tileMap.updateAll();
		}
		
		private static const SOLIDS:Array = [19, 51, 83, 8];
		private static const AUTOTILING:Array = [19, 51, 83];
		
		private static const TILE_AIR:uint = 0;
		private static const TILE_UNKNOWN:uint = 1;
		private static const TILE_EARTH:uint = 19;
		private static const TILE_ROCK:uint = 51;
		private static const TILE_STEEL:uint = 83;
		private static const TILE_BRIDGE:uint = 8;
		
		private static const COLOR_WHITE:uint = 0xffffff;
		private static const COLOR_LIGHTGRAY:uint = 0xe2e2e2;
		private static const COLOR_BLUE:uint = 0x0000ff;
		private static const COLOR_DARKGREEN:uint = 0x004400;
		private static const COLOR_GRAY:uint = 0x626262;
		private static const COLOR_BROWN:uint = 0x54371d;
		private static const COLOR_LIGHTBROWN:uint = 0x9a612e;
		private static const COLOR_ALMOSTBLACK:uint = 0x272727;
		private static const COLOR_DARKRED:uint = 0x902b21;
		private static const COLOR_DARKRED2:uint = 0x902b22;
		private static const COLOR_BEIGE:uint = 0xa48c76;
		private static const COLOR_DARKPURPLE:uint = 0x440036;
		private static const COLOR_BLUEGREEN:uint = 0x0067a2;
		private static const COLOR_STEELBLUE:uint = 0x96aece;
		private static const COLOR_PINK:uint = 0xe1a7ec;
		
		private function colorToIndex(c:uint):uint {
			//trace(c.toString(16));
			var cSolid:uint = 0x00ffffff & c;
			var r:uint;
			switch (cSolid) {
				case COLOR_WHITE:
				case COLOR_LIGHTGRAY:
				case COLOR_BLUE:
				case COLOR_ALMOSTBLACK:
				case COLOR_LIGHTBROWN:
				case COLOR_BROWN:
				case COLOR_DARKRED:
				case COLOR_DARKRED2:
				case COLOR_BEIGE:
				case COLOR_BLUEGREEN:
				case COLOR_PINK:
					r = TILE_AIR;
					break;
				case COLOR_DARKGREEN:
					r = TILE_EARTH;
					break;
				case COLOR_GRAY:
					r = TILE_ROCK;
					break;
				case COLOR_STEELBLUE:
					r = TILE_STEEL;
					break;
				case COLOR_DARKPURPLE:
					r = TILE_BRIDGE;
					break;
				default:
					trace("Warning: Unknown tile: " + cSolid.toString(16));
					r = TILE_AIR;
					break;
			}
			return r;
		}
		
		private function autoTile(tilemap:Tilemap, x:uint, y:uint):uint {
			
			var tr:Array = [21, 7, 6, 14, 15, 20, 18, 19, 5, 4, 2, 3, 13, 12, 10, 11];
			
			var t:int = tilemap.getTile(x, y);
			
			if (AUTOTILING.indexOf(t) == -1) {
				return t;
			}
			
			var b:int = 0;
			/*
			b += (tilemap.getTile(x - 1, y) == t) ? 1:0; // 1, left
			b += (tilemap.getTile(x + 1, y) == t) ? 2:0; // 2, right
			b += (tilemap.getTile(x, y - 1) == t) ? 4:0; // 4, up
			b += (tilemap.getTile(x, y + 1) == t) ? 8:0; // 8, down
			*/
			b += (tileGrid.getTile(x - 1, y)) ? 1:0; // 1, left
			b += (tileGrid.getTile(x + 1, y)) ? 2:0; // 2, right
			b += (tileGrid.getTile(x, y - 1)) ? 4:0; // 4, up
			b += (tileGrid.getTile(x, y + 1)) ? 8:0; // 8, down
			
			var result:uint = t + tr[b] - 19 + 8;
			if (result == t - 8) {
				result = t - 17 + ((x + y * 5) % 6); // variation
			}
			return result;
		}
		
		private function maybeSpawn(color:uint, x:uint, y:uint):void {
			color = color & 0xffffff;
			switch (color) {
				case COLOR_BROWN:
					add(new Tree(x * TILE_SIZE, y * TILE_SIZE, 1));
					break;
				case COLOR_LIGHTBROWN:
					add(new Tree(x * TILE_SIZE, y * TILE_SIZE, 2));
					break;
				case COLOR_ALMOSTBLACK:
					add(new Bug(x * TILE_SIZE, y * TILE_SIZE));
					add(new Bug(x * TILE_SIZE, y * TILE_SIZE));
					add(new Bug(x * TILE_SIZE, y * TILE_SIZE));
					add(new Bug(x * TILE_SIZE, y * TILE_SIZE));
					add(new Bug(x * TILE_SIZE, y * TILE_SIZE));
					break;
				case COLOR_DARKRED:
					add(new Bird(x * TILE_SIZE, y * TILE_SIZE));
					break;
				case COLOR_DARKRED2:
					add(new Bird(x * TILE_SIZE, y * TILE_SIZE, true, 1));
					break;
				case COLOR_BEIGE:
					add(new Obstacle(x * TILE_SIZE, y * TILE_SIZE));
					break;
				case COLOR_BLUEGREEN:
					add(new Orb(x * TILE_SIZE, y * TILE_SIZE));
					break;
				case COLOR_PINK:
					add(new Beam(x * TILE_SIZE, y * TILE_SIZE, tileGrid));
					break;
				default:
					break;
			}
			
			// Triggers
			if ((color & 0xffff00) == 0xff0000) {
				var id:uint = color & 0x0000ff;
				switch (id) {
					case 1:
						add(new MessageTrigger(x * TILE_SIZE, y * TILE_SIZE, "Use Z to jump, X to claw.",20,20,false ));
						break;
					case 2:
						add(new MessageTrigger(x * TILE_SIZE, y * TILE_SIZE, "If you are hurt, take shelter from the rain and your health will regenerate.", 100,100,true ));
						break;
					case 3:
						add(new MessageTrigger(x * TILE_SIZE, y * TILE_SIZE, "You found a secret place.", 100,100,true ));
						break;
					default:
						break;
				}
				
			}
		}
		
		private function positionPlayer():void {
			
			var lvlBm:Bitmap = Bitmap(new Assets.LEVEL_IMG);
			
			var x:uint;
			var y:uint;
			
			for (var i:int = 0; i < lvlBm.width; i++) {
				for (var j:int = 0; j < lvlBm.height; j++) {
					if (lvlBm.bitmapData.getPixel32(i, j) == 0xff0000ff) {
						x = i;
						y = j;
					}
				}
			}
			
			player.x = x * TILE_SIZE;
			player.y = y * TILE_SIZE;
			
			currScrX = Math.floor(player.x / SCREEN_WIDTH);
			currScrY = Math.floor(player.y / SCREEN_HEIGHT);
			
		}
		
		
		override public function update():void {
			super.update();
			
			// Update camera
			if (camOnPlayer && player.y < SCREEN_HEIGHT*(WORLD_HEIGHT_SCREENS-1)-5) {
				currScrX = Math.floor(player.x / SCREEN_WIDTH);
				currScrY = Math.floor(player.y / SCREEN_HEIGHT);
			}
			var targetX:Number = (currScrX * TILES_PER_SCREEN_X * TILE_SIZE);
			var targetY:Number = (currScrY * TILES_PER_SCREEN_Y * TILE_SIZE);
			FP.camera.x -= (FP.camera.x - targetX) * 0.3;
			FP.camera.y -= (FP.camera.y - targetY) * 0.3;
			if (Math.abs(FP.camera.x - targetX) < 2) {
				FP.camera.x = targetX;
			} 
			if (Math.abs(FP.camera.y - targetY) < 2) {
				FP.camera.y = targetY;
			} 
			
			// Debug camera control
			if (Input.pressed(Key.Q)) {
				camOnPlayer = !camOnPlayer;
			}
			if (Input.pressed(Key.NUMPAD_6)) {
				++currScrX;
			} else if (Input.pressed(Key.NUMPAD_4)) {
				--currScrX;
			} else if (Input.pressed(Key.NUMPAD_2)) {
				++currScrY;
			} else if (Input.pressed(Key.NUMPAD_8)) {
				--currScrY;
			} 
			
			
			// Rain
			var rainAm:int = rainAmount;
			while (1 < rainAm || Math.random() < rainAm) {
				var rain:Rain = new Rain((currScrX + Math.random()) * SCREEN_WIDTH, currScrY * SCREEN_HEIGHT, windSpeed + (Math.random()-0.5)*0.5);
				add(rain);
				rainAm--;
			}
			
			//Lightning
			if (Math.random() < 0.003) {
				add(new Lightning((currScrX + Math.random()) * SCREEN_WIDTH, currScrY * SCREEN_HEIGHT));
			}
			
			//Clouds
			if (FP.world.typeCount("cloud") < 1) {
				add(new Cloud( 1200,1200));
			}
			
			//Wind
			if (Math.random() < 0.001) {
				targetWind = (Math.random() -0.5) * 3;
			}
			windSpeed += (targetWind - windSpeed) * 0.01;
			
			//Messages
			if (currentMessage == null || currentMessage.finished) {
				if (messageQueue.length > 0) {
					currentMessage = messageQueue[0];
					add(currentMessage);
					messageQueue.splice(0, 1);
				}
			}
			
			//Orbs
			var orbs:Array = new Array;
			this.getClass(Orb, orbs);
			if (!won && orbs.length == 0) {
				// Game completed
				trace("all orbs collected");
				win();
				orbCounter.updateOrbs(0);
			}
			if (orbCounter.orbCount != orbs.length) {
				orbCounter.updateOrbs(orbs.length);
			}
			
			//Healthbar
			if (healthBar.health != player.health) {
				healthBar.updateHealth(player.health);
			}
			
			
			if (won) {
				
				if (winTimer > 200) {
					camera.x += (Math.random() - 0.5) * winTimer / 100;
					camera.y += (Math.random() - 0.5) * winTimer / 100;
				}
				if (winTimer == -200) {
					add(new Fade(onFaded));
				}
				
				if (winTimer < 100) {
					camera.x += (Math.random() - 0.5) * winTimer / 30;
					camera.y += (Math.random() - 0.5) * winTimer / 30;
				}
				
				winTimer--;
			}
			
		}
		
		public function showMessage(message:String):void {
			messageQueue.push(new MessagePopup(message));
		}
		
		public function orbScreenEffect():void {
			FP.screen.color = 0xff00ff;
		}
		
		public function respawnPlayer():void {
			player.init();
			positionPlayer();
		}
		
		public function win():void {
			won = true;
			winTimer = 400;
			Sounds.win.play();
		}
		
		private function onFaded():void {
			// WIN SCREEN
			trace("fading done");
			add(new End());
			remove(player);
		}
		
	}

}