package  {
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.Graphic;
	import net.flashpunk.graphics.Image;
	
	/**
	 * ...
	 * @author D Kvarfordt
	 */
	public class Particle extends Entity {
		
		private var gfx:Image;
		private var life:int;
		private var gravity:Number;
		private var wind:Boolean;
		private var colliding:Boolean;
		private var bouncyness:Number;
		private var drag:Number;
		private var vx:Number;
		private var vy:Number;
		
		
		public function Particle(x:Number, y:Number, vx:Number, vy:Number, gfx:Image, life:int = 500, gravity:Number = 0.2,
									wind:Boolean = false, colliding:Boolean = true, bouncyness:Number=0.8, drag:Number = 0.03, rotate:Boolean=true) {
			this.x = x;
			this.y = y;
			this.vx = vx;
			this.vy = vy;
			this.graphic = this.gfx = gfx;
			this.life = life;
			this.gravity = gravity;
			this.wind = wind;
			this.colliding = colliding;
			this.bouncyness = bouncyness;
			this.drag = drag;
			
			if (rotate) {
				gfx.angle = Math.random() * 360;
			}
			
			
			setHitbox(3, 3, 1, 1);
		}
		
		override public function update():void {
			super.update();
			
			vy += gravity;
			
			var windSpeed:Number = GameWorld.theWorld.windSpeed;
			if (wind) {
				if (windSpeed > 0 && vx < windSpeed) {
					vx += windSpeed/10 + (Math.random()-0.5);
				} else if (windSpeed < 0 && vx > windSpeed) {
					vx += windSpeed/10 + (Math.random()-0.5);
				}
			}
			
			vx *= (1 - drag);
			vy *= (1 - drag);
			
			if (colliding) {
				moveBy(vx, vy, "solid");
			
				if (vx > 0 && collide("solid", x + 1, y)) {
					vx *= -1;
					vx *= bouncyness;
				}
				if (vx < 0 && collide("solid", x - 1, y)) {
					vx *= -1;
					vx *= bouncyness;
				}
				if (vy > 0 && collide("solid", x, y+1)) {
					vy *= -1;
					vy *= bouncyness;
				}
				if (vy < 0 && collide("solid", x, y-1)) {
					vy *= -1;
					vy *= bouncyness;
				}
			} else {
				x += vx;
				y += vy;
			}
			
			
			if (life <= 0) {
				FP.world.remove(this);
			} else {
				life --;
			}
			
			if (life < 60 && life > 0) {
				gfx.alpha = 60.0 / Number(life);
			}
			
			
		}
		
	}

}