package  {
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.graphics.Spritemap;
	import net.flashpunk.tweens.misc.Alarm;
	import net.flashpunk.utils.Input;
	
	/**
	 * ...
	 * @author D Kvarfordt
	 */
	public class Player extends Entity {
		
		private const WIDTH:uint = 15;
		private const HEIGHT:uint = 18;
		private const SPRITE_WIDTH:uint = 32;
		private const SPRITE_HEIGHT:uint = 32;
		
		private const MOVE_FORCE:Number = 0.4;
		private const REVERSE_MOVE_FORCE:Number = 0.7;
		private const AIR_MOVE_FORCE:Number = 0.3;
		private const MAX_MOVE_SPEED:Number = 3.0;
		private const MAX_FALL_SPEED:Number = 8.0;
		private const FRICTION:Number = 0.35;
		private const JUMP_FORCE:Number = 3.8;
		private const ANTIGRAV_JUMP_TIME:int = 10;
		private const AIR_FRICTION:Number = 0.01;
		private const INVURNABILITY_TIME:int = 1;
		private const REGENERATION_TIME:int = 120;
		private const MAX_HEALTH:int = 5;
		
		private const LEFT:int = -1;
		private const RIGHT:int = 1;
		
		private var playerSpriteMap:Spritemap;
		private var claw:Claw;
		
		private var xv:Number;
		private var yv:Number;
		
		private var antiGravJumpTimer:int;
		private var invurnabilityTimer:int;
		private var onGround:Boolean;
		private var attacking:Boolean;
		private var hurt:Boolean;
		
		public var dead:Boolean;
		public var dieing:Boolean;
		public var dir:int;
		
		public var health:Number;
		private var wetness:int;
		private var regenTimer:int;
		
		//private var dieAlarm:Alarm;
		private var dieTimer:int;
		
		public function Player(x:Number, y:Number) {
			this.x = x;
			this.y = y;
			
			claw = new Claw();
			playerSpriteMap = new Spritemap(Assets.PLAYER_IMG, SPRITE_WIDTH, SPRITE_HEIGHT);
			playerSpriteMap.x = -SPRITE_WIDTH / 2;
			playerSpriteMap.y = -SPRITE_HEIGHT / 2;
			playerSpriteMap.callback = onAnimComplete;
			graphic = playerSpriteMap;
			
			initAnimations();
			
			setHitbox(WIDTH, HEIGHT, 0, 0);
			centerOrigin();
			type = "player";
			layer = -1;
			
			init();
		}
		
		public function init():void {
			playerSpriteMap.play("walk");
			xv = 0;
			yv = 0;
			antiGravJumpTimer = 0;
			onGround = false;
			attacking = false;
			hurt = false;
			dead = false;
			dieing = false;
			dieTimer = 120;
			health = MAX_HEALTH;
			wetness = 0;
		}
		
		override public function added():void {
			super.added();
			this.world.add(claw);
		}
		
		override public function removed():void {
			super.removed();
			this.world.remove(claw);
		}
		
		private function initAnimations():void {
			playerSpriteMap.add("idle", [5,6,7,8], 4,true);
			playerSpriteMap.add("stopping", [0,1,2,3,4,5], 10,false);
			playerSpriteMap.add("walk", [12,13,14,15],8);
			playerSpriteMap.add("jump", [24,25,26,27,28],10,false);
			playerSpriteMap.add("falling", [28],15,false);
			playerSpriteMap.add("attacking", [36, 37, 38, 39], 25, false);
			playerSpriteMap.add("hurt", [48], 5, false);
			playerSpriteMap.add("dead", [60], 1, true);
		}
		
		override public function update():void {
			super.update();
			
			handleInput();
			handleMovement();
			handleCollisions();
			handleAnimations();
		}
		
		private function handleInput():void {
			
			if (dieing) {
				return;
			}
			
			if (onGround) {
				if (Input.check("left")) {
					
					if (FP.sign(xv) == LEFT) {
						if (xv > -MAX_MOVE_SPEED) {
							xv -= MOVE_FORCE;
							if (xv < -MAX_MOVE_SPEED) {
								xv = -MAX_MOVE_SPEED;
							}
						} 
					} else {
						// reversing
						xv -= REVERSE_MOVE_FORCE;
					}
					
				}
				
				if (Input.check("right")) {
					
					if (FP.sign(xv) == RIGHT) {
						if (xv < MAX_MOVE_SPEED) {
							xv += MOVE_FORCE;
							if (xv > MAX_MOVE_SPEED) {
								xv = MAX_MOVE_SPEED;
							}
						}
					} else {
						// reversing
						xv += REVERSE_MOVE_FORCE;
					}
				}
				
				
				if (Input.check("jump")) {
					yv -= JUMP_FORCE;
					Sounds.jump.play();
				}
			} else {
				// NOT ON GROUND
				if (Input.check("left")) {
					
					if (xv > -MAX_MOVE_SPEED) {
						xv -= AIR_MOVE_FORCE;
						if (xv < -MAX_MOVE_SPEED) {
							xv = -MAX_MOVE_SPEED;
						}
					} 
				}
				if (Input.check("right")) {
					
					if (xv < MAX_MOVE_SPEED) {
						xv += AIR_MOVE_FORCE;
						if (xv > MAX_MOVE_SPEED) {
							xv = MAX_MOVE_SPEED;
						}
					} 
				}
				
				if (Input.check("jump") && yv < 0 && antiGravJumpTimer > 0) {
					yv -= GameWorld.GRAVITY;
				}
				antiGravJumpTimer--;
				
			}
			
			if (Input.pressed("attack")) {
				playerSpriteMap.play("attacking", true);
				claw.claw();
				attacking = true;
				Sounds.scratch.play();
			}
			
		}
		
		private function handleMovement():void {
			
			if (onGround) {
				// friction
				if (!Input.check("left") == !Input.check("right")) {
					xv *= (1 - FRICTION);
				}
			} else {
				xv *= (1 - AIR_FRICTION);
				
				if (yv < MAX_FALL_SPEED) {
					yv += GameWorld.GRAVITY;
					if (yv > MAX_FALL_SPEED) {
						yv = MAX_FALL_SPEED;
					}
				}
				if (yv < 0 && collide("solid", x, y - 1)) {
					yv = 0;
				}
			}
			
			moveBy(xv, yv, "solid");
			
			if (collide("solid", x, y + 1)) {
				onGround = true;
				antiGravJumpTimer = ANTIGRAV_JUMP_TIME;
				if (yv > 0) {
					yv = 0;
				}
			} else {
				onGround = false;
			}
			
			if (collide("solid", x + FP.sign(xv), y)) {
				xv = 0;
			}
		}
		
		private function handleCollisions():void {
			
			//Birds
			if (invurnabilityTimer <= 0 && collide("attackingbird", x, y)) {
				Sounds.meow.play();
				invurnabilityTimer = INVURNABILITY_TIME;
				playerSpriteMap.play("hurt");
				hurt = true;
				health--;
				trace("hurt, health: " + health);
			}
			
			//Falling off the level
			if (!dieing && y > GameWorld.SCREEN_HEIGHT * GameWorld.WORLD_HEIGHT_SCREENS + 100) {
				health = 0;
				dieing = true;
				Sounds.meow.play();
			}
			
			//Rain
			if (collide("rain", x, y)) {
				wetness += 30;
				if (wetness > 300) {
					wetness = 300;
				}
				regenTimer = REGENERATION_TIME;
			}
			if (wetness <= 0) {
				regenTimer--;
				if (regenTimer <= 0 && health < MAX_HEALTH) {
					health ++;
					Sounds.blip.play();
					trace("healed, health: " + health);
					regenTimer = REGENERATION_TIME;
				}
			} else {
				wetness--;
			}
			
			//Beam
			if (collide("beam", x, y)) {
				if (yv > -1) {
					yv -= 0.4;
					
				}
				xv *= 0.8;
			}
			
			invurnabilityTimer--;
			
			if (!dieing && health <= 0) {
				dieing = true;
			}
			
			if (dieing) {
				dieTimer--;
				if (dieTimer <= 0) {
					dead = true;
				}
			}
			
		}
		
		private function handleAnimations():void {
			
			claw.x = x;
			claw.y = y-5;
			
			if (xv > 0) {
				playerSpriteMap.flipped = false;
				claw.setFlipped(false);
				dir = 1;
			} else if (xv < 0) {
				playerSpriteMap.flipped = true;
				claw.setFlipped(true);
				dir = -1;
			}
			
			if (!attacking && !hurt) {
				if (onGround) {
					if (xv > 1 || xv < -1) {
						playerSpriteMap.play("walk");
					} else if (playerSpriteMap.currentAnim != "idle") {
						playerSpriteMap.play("stopping");
					}
				}
				if (!onGround) {
					if (yv < 0) {
						playerSpriteMap.play("jump");
					} else {
						playerSpriteMap.play("falling");
					}
				}
			} else if (attacking) {
				playerSpriteMap.play("attacking");
			} else if (hurt) {
				playerSpriteMap.play("hurt");
			}
			
			if (dieing) {
				playerSpriteMap.play("dead");
			}
			
		}
		
		private function onAnimComplete():void {
			if (playerSpriteMap.currentAnim == "stopping") {
				playerSpriteMap.play("idle");
			}
			if (playerSpriteMap.currentAnim == "attacking") {
				attacking = false;
			}
			if (playerSpriteMap.currentAnim == "hurt") {
				hurt = false;
			}
		}
		
		private function onDied():void {
			dead = true;
		}
	}

}